==> echion-0.1.0-cp311-cp311-macosx_10_9_x86_64.whl: echion/strings.h <==
// This file is part of "echion" which is released under MIT.
//
// Copyright (c) 2023 Gabriele N. Tornetta <phoenix1987@gmail.com>.

#pragma once

#define PY_SSIZE_T_CLEAN
#include <Python.h>
#include <unicodeobject.h>

#include <cstdint>

#include <echion/vm.h>

// ----------------------------------------------------------------------------
static char *
pybytes_to_bytes_and_size(PyObject *bytes_addr, Py_ssize_t *size)
{
    PyBytesObject bytes;

    if (copy_type(bytes_addr, bytes))
        return NULL;

    *size = bytes.ob_base.ob_size;
    if (*size < 0 || *size > (1 << 20))
        return NULL;

    char *data = new char[*size];
    if (copy_generic(((char *)bytes_addr) + offsetof(PyBytesObject, ob_sval), data, *size))
    {
        delete[] data;
        return NULL;
    }

    return data;
}

// ----------------------------------------------------------------------------
static const char *
pyunicode_to_utf8(PyObject *str_addr)
{
    PyUnicodeObject str;
    if (copy_type(str_addr, str))
        return NULL;

    PyASCIIObject &ascii = str._base._base;

    if (ascii.state.kind != 1)
        return NULL;

    const char *data = ascii.state.compact ? (const char *)(((uint8_t *)str_addr) + sizeof(ascii)) : (const char *)str._base.utf8;
    if (data == NULL)
        return NULL;

    Py_ssize_t size = ascii.state.compact ? ascii.length : str._base.utf8_length;
    if (size < 0 || size > 1024)
        return NULL;

    char *dest = new char[size + 1];
    if (copy_generic(data, dest, size))
    {
        delete[] dest;
        return NULL;
    }

    dest[size] = '\0';

    return dest;
}

==> echion-0.2.0-cp311-cp311-macosx_10_9_x86_64.whl: echion/strings.h <==
// This file is part of "echion" which is released under MIT.
//
// Copyright (c) 2023 Gabriele N. Tornetta <phoenix1987@gmail.com>.

#pragma once

#define PY_SSIZE_T_CLEAN
#include <Python.h>
#include <unicodeobject.h>

#include <cstdint>
#include <exception>
#include <string>

#include <echion/vm.h>

class StringError : public std::exception
{
    const char *what() const noexcept override
    {
        return "StringError";
    }
};

// ----------------------------------------------------------------------------
static std::unique_ptr<unsigned char[]>
pybytes_to_bytes_and_size(PyObject *bytes_addr, Py_ssize_t *size)
{
    PyBytesObject bytes;

    if (copy_type(bytes_addr, bytes))
        return nullptr;

    *size = bytes.ob_base.ob_size;
    if (*size < 0 || *size > (1 << 20))
        return nullptr;

    auto data = std::make_unique<unsigned char[]>(*size);
    if (copy_generic(((char *)bytes_addr) + offsetof(PyBytesObject, ob_sval), data.get(), *size))
        return nullptr;

    return data;
}

// ----------------------------------------------------------------------------
static std::string
pyunicode_to_utf8(PyObject *str_addr)
{
    PyUnicodeObject str;
    if (copy_type(str_addr, str))
        throw StringError();

    PyASCIIObject &ascii = str._base._base;

    if (ascii.state.kind != 1)
        throw StringError();

    const char *data = ascii.state.compact ? (const char *)(((uint8_t *)str_addr) + sizeof(ascii)) : (const char *)str._base.utf8;
    if (data == NULL)
        throw StringError();

    Py_ssize_t size = ascii.state.compact ? ascii.length : str._base.utf8_length;
    if (size < 0 || size > 1024)
        throw StringError();

    auto dest = std::string(size, '\0');
    if (copy_generic(data, dest.c_str(), size))
        throw StringError();

    return dest;
}
